---------------------------
-- lets simplify a bit..
---------------------------
local gf = Glance.Functions
local gv = Glance.Variables
local ga = Glance.Arrays
local gb = Glance.Buttons

---------------------------
-- create the button
---------------------------
gf.AddButton("Reputation","LEFT")
local btn = gb.Reputation
btn.text              = "Reputation"
btn.enabled           = true
btn.events            = {"UPDATE_FACTION"}
btn.update            = true
btn.tooltip           = true
btn.click             = true
btn.menu              = true
btn.save.perCharacter = { ["DisplayReputation"] = "Percent", ["TrackList"] = {} }
local spc = btn.save.perCharacter
btn.save.allowProfile = true

---------------------------
-- variables
---------------------------
ga.standing = {
	l = {
		"Hated",
		"Hostile",
		"Unfriendly",
		"Neutral",
		"Friendly",
		"Honored",
		"Revered",
		"Exalted"
		},
	c = {
		"|cffff0000",
		"|cffff4000",
		"|cffff8000",
		"|cffffff00",
		"|cff80ff00",
		"|cff00ff00",
		"|cff00ff80",
		"|cff02A5FD"
		},
	fc = {
		"|cffffb400",
		"|cffffff00",
		"|cff80ff00",
		"|cff00ff00",
		"|cff00ff80",
		"|cff02A5FD"
		},
		
}
gv.TrackMultiple = false

---------------------------
-- update
---------------------------
function gf.Reputation.update(self, event, arg1)
	local btn = gb.Reputation
	if btn.enabled and gv.loaded then -- loaded keeps it from launching when defined
		Glance.Debug("function","update","Reputation")
		local rep = "|cffff0000Rep"
		for factionIndex=1,GetNumFactions() do
			if select(12,GetFactionInfo(factionIndex)) then	--isWatched
				local id, name, color, standing, minVal, maxVal, currentVal, percentVal, remainingVal, isWatched, isHeader  = unpack(gf.Reputation.getFactionInfo(factionIndex))	
				if spc.DisplayReputation == "Percent" then
					rep = "Rep: "..color..percentVal
				else
					rep = "Rep: "..color..remainingVal
				end
			end
		end
		btn.button:SetText(rep)
		btn.button:SetWidth(btn.button:GetTextWidth())
	end
end


---------------------------
-- tooltip
---------------------------
function gf.Reputation.tooltip()
	Glance.Debug("function","tooltip","Reputation")
	local tooltip = gf.Tooltip
	tooltip.Title("Reputation", "GLD")
	local hasWatch = false
	local hasFriends = false
	local printedHeader = false
	local showMultiple = false
	for factionIndex=1,GetNumFactions() do
		local id, name, color, standing, minVal, maxVal, currentVal, percentVal, remainingVal, isWatched, isHeader  = unpack(gf.Reputation.getFactionInfo(factionIndex))	
		if isHeader then hasFriends = false end -- reset friend check on next header
		if select(12,GetFactionInfo(factionIndex)) then	--isWatched		
			tooltip.Line("Currently tracking "..color..name.."|r reputation: "..color..standing, "WHT")
			tooltip.Space()
			tooltip.Double("Percent Earned",color..percentVal,"WHT","GRN")
			tooltip.Double("Points Remaining",color..remainingVal,"WHT","GRN")
			hasWatch = true
			if isHeader and select(11,GetFactionInfo(factionIndex)) then -- header and hasRep means friends
				hasFriends = true
			end
		end
		if hasFriends then
			local isFriend = true
			for loop=1, #ga.standing.l do
				if standing == ga.standing.l[loop] then
					isFriend = false -- standing matches standard list, not a friend.
				end
			end
			if isFriend and not isHeader then
				if not printedHeader then --only show if rep actually has friends..
					tooltip.Space()
					tooltip.Line("Friends","GLD")
					printedHeader = true
				end
				tooltip.Double(name.." ("..color..standing.."|r)",color..percentVal.." |r("..id.."/6)","WHT","WHT")
			end
		end
		if spc.TrackList[name] then
			showMultiple = true
		end
	end
	if not hasWatch then
		tooltip.Line("You are not tracking any reputation.", "WHT")
	else			
		if showMultiple then
			tooltip.Space()
			tooltip.Line("Multiple Tracking","GLD")
			for factionIndex=1,GetNumFactions() do
				local id, name, color, standing, minVal, maxVal, currentVal, percentVal, remainingVal, isWatched, isHeader  = unpack(gf.Reputation.getFactionInfo(factionIndex))	
				if spc.TrackList[name] then
					tooltip.Double(name.." ("..color..standing.."|r)",color..percentVal,"WHT","GRN")
				end
			end
		end
	end
	
	tooltip.Space()
	tooltip.Line("Options","GLD")
	tooltip.Double("Display",spc.DisplayReputation,"WHT","GRN")	
	tooltip.Space()
	tooltip.Line("(Click to open the Reputation tab.)","WHT")
	tooltip.Line("(Right-Click to change Reputation tracking)", "WHT")
	tooltip.Line("(SHIFT Right-Click to track multiple reputations)", "WHT")
end

---------------------------
-- menu
---------------------------
function gf.Reputation.menu(level,UIDROPDOWNMENU_MENU_VALUE)
	--setInfo(disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func)
	Glance.Debug("function","menu","Reputation")
	local HEX = ga.colors.HEX
	ExpandAllFactionHeaders()
	local hName = " "
	level = level or 1
	if (level == 1) then
		gf.setInfo(nil,true,false,true,nil,HEX.white.."Reputation Options","spacer",level,nil)
		gf.setInfo(nil,nil,true,true,nil,HEX.gold.."Display","display",level,nil)
		--gf.setInfo(nil,nil,true,true,nil,HEX.gold.."TrackMultiple","multiple",level,nil)
		gf.setInfo(nil,true,false,true,nil,HEX.white.." ","spacer",level,nil)
		gf.setInfo(nil,true,false,true,nil,HEX.white.."Reputation Tracking","spacer",level,nil)
		for factionIndex=1,GetNumFactions() do
			local name = select(1,GetFactionInfo(factionIndex))
			local isHeader = select(9,GetFactionInfo(factionIndex))
			if isHeader then
				gf.setInfo(nil,nil,true,true,nil,HEX.gold..name,name,level,nil)
			end
		end
	end
	if (level == 2) then
		for factionIndex=1,GetNumFactions() do
			local id, name, color, standing, minVal, maxVal, currentVal, percentVal, remainingVal, isWatched, isHeader = unpack(gf.Reputation.getFactionInfo(factionIndex))	
			local val = ""
			if spc.DisplayReputation == "Percent" then --use percent or remaining value
				val = percentVal
			else
				val = remainingVal
			end
			if isHeader then hName = name end -- set the next header
			if hName == UIDROPDOWNMENU_MENU_VALUE and ((not isHeader) or (select(11,GetFactionInfo(factionIndex)))) then -- name match and (not header or hasRep)
				local ks, nr = false, false
				if gv.TrackMultiple then -- tracking multiple, change to checkboxes
					ks = true
					nr = true
				end
				if select(11,GetFactionInfo(factionIndex)) then -- hasRep, add a border
					gf.setInfo(nil,true,false,true,nil,HEX.gold.."     ----------------------------------","spacer",level,nil)
				end
				local checked, func, text
				text = color..standing.." ("..val..") |r"..name -- text
				
				if gv.TrackMultiple then -- tracking multiple, func updates tracklist
					if spc.TrackList[name]==true then checked = true else checked = false end
					func = function() if spc.TrackList[name] then spc.TrackList[name]=false else spc.TrackList[name]=true end; end;
				else -- tracking single, func changes watch
					if isWatched then
						checked = true
						func = function() SetWatchedFactionIndex(0); gf.Reputation.update() end;
					else
						checked = false
						func = function() SetWatchedFactionIndex(factionIndex); gf.Reputation.update() end;
					end
				end
				--disabled,isTitle,hasArrow,notCheckable,checked,text,value,level,func,icon,keepShown,notRadio
				gf.setInfo(nil,false,false,false,checked,text,hName,level,func,nil,ks,nr)
				if select(11,GetFactionInfo(factionIndex)) then -- hasRep, add a border
					gf.setInfo(nil,true,false,true,nil,HEX.gold.."     ----------------------------------","spacer",level,nil)
				end
			end
		end
		if UIDROPDOWNMENU_MENU_VALUE == "display" then
			local checked, func
			if spc.DisplayReputation=="Percent" then checked = true else checked = false end
			func = function() spc.DisplayReputation="Percent"; gf.Reputation.update() end;
			gf.setInfo(nil,false,false,false,checked,"Percent","Percent",level,func)
			
			if spc.DisplayReputation=="Remaining" then checked = true else checked = false end
			func = function() spc.DisplayReputation="Remaining"; gf.Reputation.update() end;
			gf.setInfo(nil,false,false,false,checked,"Remaining","Remaining",level,func)
		end
	end
end

---------------------------
-- click
---------------------------
function gf.Reputation.click(self, button, down)
	Glance.Debug("function","click","Reputation")
	if button == "LeftButton" then
		ToggleCharacter("ReputationFrame")
	elseif button == "RightButton" then
		if IsShiftKeyDown() then
			gv.TrackMultiple = true
		else
			gv.TrackMultiple = false
		end
	end
end

---------------------------
-- get faction info
---------------------------
function gf.Reputation.getFactionInfo(factionIndex)
	local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, hasRep, isWatched, isChild, factionID, hasBonusRepGain, canBeLFGBonus = GetFactionInfo(factionIndex)
	local fsText, fsColor, fsPercent, fsRemaining;
	local friendID, friendRep, friendMaxRep, friendName, friendText, friendTexture, friendTextLevel, friendThreshold, nextFriendThreshold = GetFriendshipReputation(factionID);
	if (friendID ~= nil) then
		if ( nextFriendThreshold ) then
			barMin, barMax, barValue = friendThreshold, nextFriendThreshold, friendRep;
		else
			barMin, barMax, barValue = 0, 1, 1;
		end
		fsText = friendTextLevel;
		fsColor = ga.standing.fc[standingID];
	else
		fsText = ga.standing.l[standingID];
		fsColor = ga.standing.c[standingID];
	end	
	fsPercent = gf.Reputation.calculate(barMin,barValue,barMax)
	fsRemaining = barMax - barValue
	return {standingID, name, fsColor, fsText, barMin, barMax, barValue, fsPercent, fsRemaining, isWatched, isHeader}
end

---------------------------
-- calculate rep %
---------------------------
function gf.Reputation.calculate(bv,ev,tv)
	local pct
	if tv > 0 then
		pct = math.floor(((ev-bv)/(tv-bv))*100)
	else
		pct = -(math.floor(((ev-tv)/(tv-bv))*100))
	end
	return pct.."%"
end

---------------------------
-- load on demand
---------------------------
if gv.loaded then
	gf.Enable("Reputation")
end
